from typing import Iterable, Generator
from ..dtos.reputation_item import ReputationItem
from ....shared.aws.dynamodb import DynamoDbTable


class ReputationDbRepository(object):
    """
    Repository class to interact with reputation dynamodb.
    """

    def __init__(self, table_name, session=None, config=None, endpoint_url=None):
        """
        Sets up a boto3 client to interact with database.

        @param table_name: Name of the dynamodb table to use.
        @param session: the boto3 session to reuse if provided.
        @param config: The botocore config.
        """

        self.table = DynamoDbTable(
            boto_session=session,
            table_name=table_name,
            model=ReputationItem,
            boto_config=config,
            endpoint_url=endpoint_url,
        )

    def update(self, items):
        """
        Updates a list of items.

        @param items: Items to put in the cache. ReputationItems objects expected. Each must have a sha256 populated.
        """
        self.table.conditional_write(
            items=items, must_be_greater_field="version", value_ignorelist={None, ""}
        )

    def update_unconditional(self, items):
        """
        Updates a list of items.

        @param items: Items to put in the cache. ReputationItems objects expected. Each must have a sha256 populated.
        """
        self.table.write(items=items, value_ignorelist={None, ""})

    def delete(self, sha256s):
        """
        Deletes a list of items.

        @param sha256s: A list of the sha256 pKeys of the records to delete.
        """
        self.table.delete(pkeys=sha256s)

    def get(self, sha256s: Iterable[str]) -> Generator[ReputationItem, None, None]:
        """
        A generator of ReputationItems from cache.

        @param sha256s: A list of sha256 hashes to get.
        """
        for item in self.table.get(keys=[(pkey,) for pkey in sha256s]):
            yield item

    def get_md5s(self, md5s):
        """
        A generator of ReputationItems from cache.

        @param md5s: a list or set of md5 items to get.
        """

        for item in self.table.query_on_index(index="md5", column="md5", values=md5s):
            yield item
